function [fcnt fea] = ExtFeature(pixmap)

[h w] = size(pixmap);
fea = [0; 0; 0; 0; 0];
r = 2;
c = 2;
fcnt = 0;

% uniform sample of seeds
while r < h
    c = 2;
    while c < w
        % if seed on the stroke, extract feature
        if pixmap(r, c) == 0
            pos = [r c];    % seed's position - r: row, c: column
            oldpos = [r c]; % seed's previous position(s)
            a = 1;          % length of minor axis - temporary storage
            minorlen = 1;   % length of minor axis - final result
            angidx = 0;     % angle (orientation) index. 1 = 10 degrees
            % find minor axis and orientation
            while a > 0
                for angidx=0:17
                    p1 = pos + [-a*sin(pi/18*angidx) a*cos(pi/18*angidx)];
                    p1 = round(p1);         % end point 1
                    p2 = pos + pos - p1;    % end point 2, opposite direction of end point 1
                    % check if point is out of boundary
                    if pos(1) < 1 || pos(2) < 1 || pos(1) > h || pos(2) > w
                        a = -1;
                        break;
                    end;
                    if pixmap(p1(1), p1(2)) == 0 % check point 1
                        if pixmap(p2(1), p2(2)) == 0 % check point 2 - opposite of point 1
                        else
                            % adjust seed position if only p2 is not in stroke
                            pos = pos - [sin(pi/18*angidx) -cos(pi/18*angidx)];
                            pos = round(pos);
                            % if the seed goes back to old position, stop adjusting seed position
                            diffpos = oldpos - repmat(pos, size(oldpos,1), 1);
                            mindiff = min(abs(diffpos(:,1)) + abs(diffpos(:,2)));
                            if mindiff == 0
                                minorlen = a;
                                a = -1;
                            else
                                a = a-1;
                                oldpos(size(oldpos,1)+1,1:2) = pos;
                            end;
                            break;
                        end;
                    else
                        if pixmap(p2(1), p2(2)) == 0 % check point 2 - opposite of point 1
                            % adjust seed position if only p1 is not in stroke
                            pos = pos + [sin(pi/18*angidx) -cos(pi/18*angidx)];
                            pos = round(pos);
                            % if the seed goes back to old position, stop adjusting seed position                           
                            diffpos = oldpos - repmat(pos, size(oldpos,1), 1);
                            mindiff = min(abs(diffpos(:,1)) + abs(diffpos(:,2)));
                            if mindiff == 0
                                minorlen = a;
                                a = -1;
                            else
                                a = a-1;
                                oldpos(size(oldpos,1)+1,1:2) = pos;
                            end;
                            break;
                        else
                            % not p1 nor p2 is in stroke, minor axis fixed
                            a = -1;
                            break;
                        end;
                    end;
                end;
                a = a + 1;
            end;
            
            % find major axis length & orientation [0 pi]
            b = [0 0 0]; % major axis lengths with a bit orientation differences
            orien = [angidx+10 angidx+9 angidx+8]; % orientations - +, normal,-
            if orien(1) > 17, orien(1) = orien(1) - 18;, end;
            if orien(2) > 17, orien(2) = orien(2) - 18;, end;
            if orien(3) > 17, orien(3) = orien(3) - 18;, end;
            begin = true;
            % find the longest major axis length possible with minor
            % adjustments in orientation
            while begin || max(b) > b(2)
                prevb = b;
                prevo = orien;
            
                % original orientation
                if begin                
                    b(2) = findb(pos, orien(2), h, w, pixmap);
                elseif prevb(1) == max(prevb)
                    b(2) = prevb(1);
                    orien(2) = prevo(1);
                elseif prevb(3) == max(prevb)
                    b(2) = prevb(3);
                    orien(2) = prevo(3);
                else
                    break;
                end;
                
                % - orientation
                if begin
                    b(3) = findb(pos, orien(3), h, w, pixmap);
                elseif prevb(3) == max(prevb)
                    orien(3) = prevo(3) - 1;
                    if orien(3) < 0, orien(3) = orien(3) + 18;, end;
                    b(3) = findb(pos, orien(3), h, w, pixmap);
                end;
                
                % + orientation
                if begin
                    b(1) = findb(pos, orien(1), h, w, pixmap);
                elseif prevb(1) == max(prevb)
                    orien(1) = prevo(1) - 1;
                    if orien(1) < 0, orien(1) = orien(1) + 18;, end;
                    b(1) = findb(pos, orien(1), h, w, pixmap);
                end;
                
                begin = false;
            end;
            % record seed info
            fcnt = fcnt + 1;
            if b(1) > b(2)
                if b(1) > b(3)
                    fea(1:5,fcnt) = [pos(1) pos(2) 2*b(1) pi/18*orien(1) 2*minorlen];                    
                else
                    fea(1:5,fcnt) = [pos(1) pos(2) 2*b(3) pi/18*orien(3) 2*minorlen];  
                end;
            elseif b(3) > b(2)
                fea(1:5,fcnt) = [pos(1) pos(2) 2*b(3) pi/18*orien(3) 2*minorlen];
            else
                fea(1:5,fcnt) = [pos(1) pos(2) 2*b(2) pi/18*orien(2) 2*minorlen];
            end;
                        
        end;
        c = c + 5;
    end;
    r = r + 5;
end;


function b = findb(pos, orien, h, w, pixmap)
b = 0;
while true
    p1 = pos + [-b*sin(pi/18*orien) b*cos(pi/18*orien)];
    p1 = round(p1);
    if p1(1) < 1 || p1(1) > h || p1(2) < 1 || p1(2) > w || pixmap(p1(1), p1(2)) == 1
        break;
    end;
    p2 = pos + pos - p1;
    if p2(1) < 1 || p2(1) > h || p2(2) < 1 || p2(2) > w || pixmap(p2(1), p2(2)) == 1
        break;
    end;
    b = b + 1;
end;
if b > 0, b = b - 1;, end;



