function [b finalpic] = execSTD(unknown, templates, unknowna, templatesa, templatesnum, scoresa, scoresa2, msghandle, orderhandle)
% unknow - character features, 5xN array
% temmplates - template feature cell array, 1xM, each is a 5xN array
% unknowna - axes to draw unknown
% templatesa - axes to draw template feature array
% scoresa - axes to draw score bar graph
% scoresa2 - axes to annotate score bar graphs
% msghandle - text message handle

T = 100;        % Maximum iterations
t = 0;          % Current iteration
alpha = 0.5;    % Learning rate
s = 1;          % Current scaling factor of ellipse neighborhood
sfac = 0.99;     % Shrink facotr of scaling factor
picw = 100;     % Picture width
pich = 100;     % Picture height

% Colors
c_blue = [0 0 1];
c_red = [1 0 0];
c_green = [0 1 0];
c_yellow = [0.9 0.9 0];
c_purple = [1 0 1];
c_cyan = [0 1 1];

% image data to be painted for each template
imgdata = cell([size(templates, 1) 1]);
% SOM networks of templates
networks = cell([size(templates, 1) 1]);
% painting axes index of each template
axesidx = [0];
imagelist = cell([size(templates, 1) 1]);
% score data
scores = zeros([size(templates, 1) 1]);

% init plot data
blank_pic = ones(pich, picw);
blank_pic = cat(3, blank_pic, blank_pic, blank_pic);

% Initialize data
for i=1:size(templatesa, 1)
    imgdata{i} = blank_pic;
    set(gcf, 'CurrentAxes', templatesa{i});
    imagelist{i} = image(imgdata{i});
    set(gca, 'YDir', 'reverse');
    set(gca, 'XTick', []);
    set(gca, 'YTick', []);
end;

for i=1:size(templates, 1)
    % set which axes to plot for the template
    if i > size(templatesa, 1)
        m = floor(size(templates, 1) / size(templatesa, 1));
        axesidx(i) = i - size(templatesa, 1) * m;
        if axesidx(i) == 0, axesidx(i) = size(templatesa, 1);, end;            
    else
        axesidx(i) = i;
    end;
    
    % copy templates to SOM network
    networks{i} = templates{i};

    imgdata{i} = blank_pic;
    % plot unknown
    for j=1:size(unknown,2)    
        imgdata{i}(round(unknown(1,j)), round(unknown(2,j)), 1:3) = c_red;
    end;
    % plot template
    template = templates{i};
    for j=1:size(template, 2)    
        imgdata{i}(round(template(1,j)), round(template(2,j)), 1:3) = c_blue;
    end;    
end;

for t=1:T
    set(msghandle, 'String', ['Time ', int2str(t)]);
    scores = zeros([size(templates, 1) 1]);
    for i=1:size(templates, 1)
        % randomly pick one feature from unknown pattern
        n = round(size(unknown,2)*rand);
        if n == 0
            n = 1;
        end;
        
        % train SOM
        % closest neuron to the feature
        nidx = findneuron(unknown(1:4,n), networks{i});        
        cp = networks{i}(1:4, nidx);
        imgdata{i}(round(cp(1,1)), round(cp(2,1)), 1:3) = c_cyan;
        
        % modify neurons within neighborhood
        % w is the vector of major axis
        w = [unknown(3,n)*cos(unknown(4,n))/2; -unknown(3,n)*sin(unknown(4,n))/2];
        % examing all neurons and only modify those within the neighborhood
        for k=1:size(networks{i}, 2)            
            v = networks{i}(1:2, k)-cp(1:2);        
            % projection onto major axis
            vw = v'*w*w/(w'*w);
            r = vw'*vw;
            c = sqrt(v'*v - vw'*vw);
            d = (r/((unknown(3,n)/2)*s))^2+(c/(unknown(5,n)/2)*s)^2;        
            if d < 1
                %networks{i}(1:2,k) = networks{i}(1:2,k) + alpha*(unknown(1:2,n)-cp(1:2,1));
                networks{i}(1:4,k) = networks{i}(1:4,k) + alpha*(unknown(1:4,n)-networks{i}(1:4,k));
            end;
            % draw a cross sign
            imgdata{i}(round(cp(1,1)), round(cp(2,1)), 1:3) = c_green;
            imgdata{i}(round(cp(1,1))-1, round(cp(2,1)), 1:3) = c_green;
            imgdata{i}(round(cp(1,1))+1, round(cp(2,1)), 1:3) = c_green;
            imgdata{i}(round(cp(1,1)), round(cp(2,1))-1, 1:3) = c_green;
            imgdata{i}(round(cp(1,1)), round(cp(2,1))+1, 1:3) = c_green;
        end;
        
        % random neuron
        n = round(size(networks{i},2)*rand);
        if n == 0
            n = 1;
        end;
        imgdata{i}(round(networks{i}(1, n)), round(networks{i}(2, n)), 1:3) = c_cyan;        
        nidx = findneuron(networks{i}(1:4,n), unknown); 
        networks{i}(1:2,n) = networks{i}(1:2,n) + alpha*(unknown(1:2,nidx)-networks{i}(1:2,n));
        imgdata{i}(round(networks{i}(1,n)), round(networks{i}(2,n)), 1:3) = c_green;
        imgdata{i}(round(networks{i}(1,n))-1, round(networks{i}(2,n)), 1:3) = c_green;
        imgdata{i}(round(networks{i}(1,n))+1, round(networks{i}(2,n)), 1:3) = c_green;
        imgdata{i}(round(networks{i}(1,n)), round(networks{i}(2,n))-1, 1:3) = c_green;
        imgdata{i}(round(networks{i}(1,n)), round(networks{i}(2,n))+1, 1:3) = c_green;
        
        %for k=1:size(networks{i}, 2)
        %    scores(i) = scores(i) + findmin(networks{i}(1:2,k), unknown);
        %end;
        
        for k=1:size(networks{i}, 2)
            scores(i) = scores(i) + (networks{i}(1:2,k)-templates{i}(1:2,k))'*(networks{i}(1:2,k)-templates{i}(1:2,k));
        end;
        scores(i) = scores(i)/size(networks{i},2);
        set(templatesnum{axesidx(i)}, 'String', int2str(i));        
        set(imagelist{axesidx(i)}, 'CData', imgdata{i});
        drawnow;        
    end;
    tlabel = 1:size(scores, 1);
    tlabel = cat(2, tlabel', scores);
    tlabel = sortrows(tlabel, 2);
    sortedtsc = [int2str(tlabel(1,1)) ' - ' num2str(tlabel(1,2), '%11.4g')];
    for i=2:size(scores, 1)
        sortedtsc = strvcat(sortedtsc, [int2str(tlabel(i,1)) ' - ' num2str(tlabel(i,2), '%11.4g')]);
    end;
    set(orderhandle, 'String', sortedtsc);
    set(gcf, 'CurrentAxes', scoresa);
	cla;
    bar(scoresa,scores);
    ylabel(scoresa, 'Scores');
    xlabel(scoresa, 'Template');
    set(scoresa2,'YAxisLocation','right','Color','none','XTickLabel',[]);
    set(scoresa2,'XLim',get(scoresa,'XLim'),'Layer','top');
    set(scoresa2,'YLim',get(scoresa,'YLim'),'Layer','top');
    set(gcf, 'CurrentAxes', scoresa2);
    cla;
    for i=1:size(scores,1)
        text(i,scores(i), num2str(scores(i), '%11.4g'),'HorizontalAlignment','center','VerticalAlignment','bottom','FontSize',8);
    end;
    set(gcf,'PaperPositionMode','auto');
end;

% find best matched template
b = 1;
for i=2:size(scores, 1)
    if scores(i) < scores(b), b = i;, end;
end;
finalpic = imgdata{b};

function idx = findneuron(fea, net)
% find the neuron in net which is closest to fea in distance (uses 4D feature)
% fea - a feature, 5x1
% net - the SOM network, 5x1 size neurons
idx = 1;
mindist = (fea-net(1:4,1))'*(fea-net(1:4,1));
for i=2:size(net, 2)
    dist = (fea-net(1:4,i))'*(fea-net(1:4,i));
    if dist < mindist
        idx = i;
        mindist = dist;
    end;
end;

function minscr = findmin(stdfea, unknown)
% find the minimum score of stdfea to the unknown pattern (uses 2D feature)
% stdfea - a feature in a standard template pattern, 5x1
% unknown - unknow pattern, 5x1 size features
minscr = (stdfea-unknown(1:2,1))'*(stdfea-unknown(1:2,1));
for i=1:size(unknown,2)
    scr = (stdfea-unknown(1:2,i))'*(stdfea-unknown(1:2,i));
    if scr < minscr
        minscr = scr;
    end;
end;
minscr = sqrt(minscr);

function minscr = findmin2(stdfea, stdfeaT)
% find the minimum score of stdfea to the unknown pattern (uses 2D feature)
% stdfea - a feature in a standard template pattern, 5x1
% unknown - unknow pattern, 5x1 size features
minscr = (stdfea-stdfeaT(1:2,1))'*(stdfea-stdfeaT(1:2,1));
for i=1:size(stdfeaT,2)
    scr = (stdfea-stdfeaT(1:2,i))'*(stdfea-stdfeaT(1:2,i));
    if scr < minscr
        minscr = scr;
    end;
end;
minscr = sqrt(minscr);
